/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMHTMLElement.idl"

/**
 * The nsIDOMHTMLFormElement interface is the interface to a [X]HTML
 * form element.
 *
 * This interface is trying to follow the DOM Level 2 HTML specification:
 * http://www.w3.org/TR/DOM-Level-2-HTML/
 *
 * with changes from the work-in-progress WHATWG HTML specification:
 * http://www.whatwg.org/specs/web-apps/current-work/
 */

[scriptable, uuid(5e49bff8-fb61-41e3-b6a9-2017865a6d74)]
interface nsIDOMHTMLFormElement : nsIDOMHTMLElement
{
           attribute DOMString            acceptCharset;
           attribute DOMString            action;
           attribute DOMString            autocomplete;
           attribute DOMString            enctype;
           attribute DOMString            encoding;
           attribute DOMString            method;
           attribute DOMString            name;
           attribute boolean              noValidate;
           attribute DOMString            target;

  readonly attribute nsIDOMHTMLCollection elements;
  readonly attribute long                 length;

  void                      submit();
  void                      reset();
  boolean                   checkValidity();
};
